#!/bin/sh
# Ruari Oedegaard
# hacked for puppy by 01micko

available () {
  command -v "$1" >/dev/null 2>&1
}

# Checks if the same Vivaldi version is already installed
checkversion () {
  if [ -e "/opt/$(echo $VIVALDI_STREAM | sed 's/\-stable//')/VERSION_$VIVALDI_VERSION" ]; then
    echo "$VIVALDI_STREAM ($VIVALDI_VERSION) is already installed; exiting"
    sleep 2
    exit 0
  else
    # uninstall older version
	is_pet_installed vivaldi
	if [ $? -eq 1 ];then
	  is_sfs_loaded vivaldi
	fi
  fi
}


# Binutils or libarchive (bsdtar) is needed for this script to run
if ! available ar; then
  if ! available bsdtar; then
    echo "You must install GNU Binutils or libarchive to use this script" >&2
    exit 1
  fi
  # source the special func lib that contains an 'ar' wrapper with bsdtar
  . /usr/share/dl_browser/s.browser.func
fi

# if we get here source browser.func
. /usr/share/dl_browser/browser.func

# package type
PKG_TYPE=${PKG_TYPE:-pet}
echo "Package type selected is $PKG_TYPE"
sleep 3

# Check if this is being run as a self extractor
if [ -z "$1" ]; then
  if [ $(sed '1,/^exit$/d' "$0" | head -n 1 | wc -l) = "1" ]; then
    VIVALDI_DEB=$(mktemp -t vivaldi-deb.XXXXXX)
    sed '1,/^exit$/d' "$0" > "$VIVALDI_DEB"
    DESTROY_DEB=Y
  fi
fi

# Check if automatic download has been selected
while [ 0 ]; do
  if [ "$1" = "-s" -o "$1" = "--snapshot" ]; then
    VIVALDI_STREAM=vivaldi-snapshot
    DESTROY_DEB=Y
    shift 1
  elif [ "$1" = "-p" -o "$1" = "--beta" ]; then
    VIVALDI_STREAM=vivaldi-beta
    DESTROY_DEB=Y
    shift 1
  elif [ "$1" = "-p" -o "$1" = "--stable" ]; then
    VIVALDI_STREAM=vivaldi-stable
    DESTROY_DEB=Y
    shift 1
  else
    break
  fi
done

# Run some checks to see if a proper package has been selected
if [ -z "$DESTROY_DEB" -a -n "$1" ]; then
  if ! echo "$1" | grep -q '[Vv]ivaldi.*\.deb$'; then
    echo "$1 is not named like an Vivaldi Debian package" >&2
    exit 1
  fi
  if [ -r "$1" ]; then
    VIVALDI_DEB="$1"
    DESTROY_DEB=N
    VIVALDI_AUTO_FETCH=N
  fi
fi

# Extract information from control file
if [ -n "$VIVALDI_DEB" ]; then
  VIVALDI_STREAM_VERSION_DEBARCH=$(ar -p "$VIVALDI_DEB" control.tar.gz | tar -xzOf- ./control | sed -n 's/^Package: //p;s/^Version: //p;s/^Architecture: //p')
  if [ -z "$VIVALDI_STREAM_VERSION_DEBARCH" ]; then
    echo "Could not extract the package name and architecture from the control file" >&2
    exit 1
  else
    VIVALDI_STREAM=$(echo $VIVALDI_STREAM_VERSION_DEBARCH | cut -d' ' -f1)
    VIVALDI_VERSION=$(echo $VIVALDI_STREAM_VERSION_DEBARCH | cut -d' ' -f2)
    DEBARCH=$(echo $VIVALDI_STREAM_VERSION_DEBARCH | cut -d' ' -f3)
  fi

  # Set the architecture
  case "$DEBARCH" in
    amd64) ARCH=x86_64 ;;
        *) ARCH=$DEBARCH ;;
  esac
  
  checkversion
else
  VIVALDI_STREAM=${VIVALDI_STREAM:-vivaldi-stable}
  DESTROY_DEB=Y
  # Set architecture information
  ARCH=$(uname -m | sed 's/i.86/i386/')
  case "$ARCH" in
    x86_64) DEBARCH=amd64 ;;
      i386) DEBARCH=$ARCH ;;
         *) echo "The architecture $ARCH is not supported." >&2 ; exit 1 ;;
  esac

  # Make sure we have wget or curl
  if available wget; then
    SILENT_DL="wget -qO-"
    LOUD_DL="wget"
    DL_OUTPUT="-O"
  elif available curl; then
    SILENT_DL="curl -s"
    LOUD_DL="curl"
    DL_OUTPUT="-o"
  else
    echo "Install wget or curl" >&2
    exit 1
  fi

  # Work out the latest Vivaldi version
  VIVALDI_VERSION=$($SILENT_DL http://repo.vivaldi.com/archive/deb/dists/stable/main/binary-$DEBARCH/Packages.gz | gzip -d | grep -A6 -x "Package: $VIVALDI_STREAM" | sed -n "/Version/s/.* //p" | sort | tail -n 1)

  # Error out if $VIVALDI_VERISON is unset, e.g. because previous command failed
  if [ -z "$VIVALDI_VERSION" ]; then
    echo "Could not work out the latest version of $VIVALDI_STREAM for $ARCH; exiting" >&2
    exit 1
  fi

  checkversion

  VIVALDI_DEB=$(mktemp -t vivaldi-deb.XXXXXX)
  $LOUD_DL http://repo.vivaldi.com/archive/deb/pool/main/${VIVALDI_STREAM}_${VIVALDI_VERSION}_${DEBARCH}.deb $DL_OUTPUT $VIVALDI_DEB
  if ! [ "$?" = 0 ]; then
    echo "Download failed!" >&2
    exit 1
  fi
fi

# Stop the script as soon as there is a problem
set -eu

# Extract files from the Debian package to a temporary location
VIVALDI_FILES=$(mktemp -d -t vivaldi-files.XXXXXX)
printf "\nUncompressing Vivaldi ...\n"
ar p "$VIVALDI_DEB" data.tar.xz | xz -d | tar -xf- -C "$VIVALDI_FILES" \
  --exclude="./usr/share/lintian"   --exclude="./usr/share/xfce4" --exclude="./usr/share/menu" --exclude="./etc"

# Remove the last part of stable stream name
VIVALDI_STREAM=$(echo $VIVALDI_STREAM | sed 's/\-stable//')

# Record the version number in the package
cd "$VIVALDI_FILES"
touch opt/$VIVALDI_STREAM/VERSION_$VIVALDI_VERSION

# Symlink desktop environment icons
for png in opt/$VIVALDI_STREAM/product_logo_*.png; do
  pngsize="${png##*/product_logo_}"
  mkdir -p usr/share/icons/hicolor/${pngsize%.png}x${pngsize%.png}/apps
  (
    cd usr/share/icons/hicolor/${pngsize%.png}x${pngsize%.png}/apps/
    ln -s ../../../../../../opt/$VIVALDI_STREAM/product_logo_${pngsize} $VIVALDI_STREAM.png
  )
done

# fix.desktop
sed -i -e 's/Icon=.*/Icon=vivaldi.svg/' \
		-e "s/Exec=/Exec=run-as-spot /" \
		-e 's/ %U//' \
		-e 's/Categories=.*/Categories=WebBrowser;/' \
		usr/share/applications/*.desktop

# Correct the Vivaldi sandbox permissions
chmod 4755 opt/$VIVALDI_STREAM/vivaldi-sandbox

# package as pet or sfs
( 
cd ../ # /tmp
if [ "$PKG_TYPE" = "pet" ]; then
  p=${VIVALDI_STREAM}-${VIVALDI_VERSION}-${ARCH}
  mv "$VIVALDI_FILES" ${p}
  chmod 755 $p
  DESC="Vivaldi is a web browser based on the blink engine"
  DEPS=""
  CAT=Internet
  echo "Please wait while constructing ${p}.pet ..."
  dir2pet -x -s -w="$DESC" -d="$DEPS" -c="$CAT" -p=${p} 2>&1 >/dev/null
else # sfs
  chmod 755 "$VIVALDI_FILES"
  mksquashfs "$VIVALDI_FILES" ${VIVALDI_STREAM}-${VIVALDI_VERSION}-${ARCH}.sfs
fi
)

# Remove temporary files
cd - >/dev/null
rm -rf "$VIVALDI_FILES"
if [ "$DESTROY_DEB" = "Y" ]; then
  rm "$VIVALDI_DEB"
fi

# And ... we're done! ;)
echo 'Vivaldi was successfully packaged'
exit
